#ifndef glas_concept_concept_hpp
#define glas_concept_concept_hpp

#include <glas/concept/expression_concept.hpp>
#include <boost/mpl/vector.hpp>
#include <boost/mpl/eval_if.hpp>
#include <boost/type_traits/is_const.hpp>
#include <boost/type_traits/remove_const.hpp>
#ifdef GLAS_COMPLEX
#  include <complex>
#endif

namespace glas {

  namespace detail {

    template <class T>
    struct concept_default 
    { typedef typename T::concept_type type ; } ;

    template <>
    struct concept_default<float>
    { typedef boost::mpl::vector< expression_concept > type ; } ;

    template <>
    struct concept_default<double>
    { typedef boost::mpl::vector< expression_concept > type ; } ;

#ifdef GLAS_COMPLEX
    template <class T>
    struct concept_default< std::complex<T> >
    { typedef boost::mpl::vector< expression_concept > type ; } ;
#endif

  } ;

  // It T is a const type, remove the const because concept is
  // specialized for non-const types only.
  template <class T>
  struct concept
  : boost::mpl::eval_if< boost::is_const<T>
                   , concept<typename boost::remove_const<T>::type>
                   , detail::concept_default<T>
                   >
  {} ;

}

#endif // glas_meta_concept_hpp

